/*
   +----------------------------------------------------------------------+
   | PHP Version 4                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2007 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: Shane Caraveo             <shane@caraveo.com>               |
   |          Colin Viebrock            <colin@easydns.com>               |
   |          Hartmut Holzgraefe        <hholzgra@php.net>                |
   |          Wez Furlong               <wez@thebrainroom.com>            |
   +----------------------------------------------------------------------+
 */
/* $Id: c_ext.c,v 1.30.2.4.2.2 2007/01/01 09:46:40 sebastian Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef PHP_WIN32
#define _WINNLS_
#endif

#include "php.h"
#include "php_c_ext.h"
#include "config_db.h"
#include "confd_defs.h"
#include "db_com.h"
#include "xstr.h"
#include "getopt.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <bufutil.h>
#include <stdio.h>
#include <unistd.h>

function_entry c_ext_functions[] = {
	PHP_FE(conf_get,NULL)
	PHP_FE(conf_get_next,NULL)
	PHP_FE(conf_iterate,NULL)
	PHP_FE(conf_set,NULL)
	PHP_FE(conf_set_buffer,NULL)
	PHP_FE(conf_save,NULL)
	{NULL, NULL, NULL}
};


zend_module_entry c_ext_module_entry = {
	STANDARD_MODULE_HEADER,
	"c_ext",
	c_ext_functions,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NO_VERSION_YET,
	STANDARD_MODULE_PROPERTIES,
};

#ifdef COMPILE_DL_c_ext
ZEND_GET_MODULE(c_ext)
#endif

extern char *optarg;
extern int optind, opterr, optopt;


PHP_FUNCTION(conf_get)
{
		dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
		char *val = NULL, *err=NULL,*get_buffer;
		int get_buffer_len;


		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &get_buffer,  &get_buffer_len) == SUCCESS)
		{
			if( dbcom_get_buffer(state,get_buffer,  &val, &err))
					val = "Get Failed\n";
			else
			{
					if (val == NULL)
							val = "";
			}

		}
		else
		{
			printf("\nZEND parsing failed\n");
			RETURN_FALSE;
		}
		RETURN_STRING(val,1);
}



PHP_FUNCTION(conf_get_next)
{
    dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
    char *val = NULL, *err=NULL,*get_buffer;
	int get_buffer_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &get_buffer,  &get_buffer_len) == SUCCESS)
	{
		if( dbcom_getnext_buffer(state, get_buffer,  &val, &err))
			val = "Get Failed\n";
    }
    else
	{
		printf("\nZEND parsing failed\n");
		RETURN_FALSE;
	}
		RETURN_STRING(val,1);
}


PHP_FUNCTION(conf_iterate)
{
    dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
    char *val = NULL, *err=NULL,*get_options,*get_value,ch;
    int flags= 0,opt_len,val_len,i=1;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &get_options,&opt_len,&get_value,&val_len) == SUCCESS)
	{
		while (i <opt_len)
		{
		   ch = get_options[i++];
		   switch (ch)
		   {
				case 'c':
				 flags= flags|DBCOM_ITERATE_CHILDREN;
			 continue;
				case 'a':
				 flags= flags|DBCOM_ITERATE_ALL;
			 continue;
				case 'v':
				 flags= flags|DBCOM_ITERATE_VALUE;
			 continue;
				case 'r':
			 flags= flags|DBCOM_ITERATE_RECURSE;
			 continue;
				case '?':
					 printf("usage: conf_iterate [-arvc] <name>\n");
			 exit(0);
		   }
	  	}
		dbcom_iterate_buffer(state, get_value, flags,  &val, &err);
		if (err)
			val = "Iterate failed\n";
		if (val == NULL)
			val = "";

		RETURN_STRING(val,1);
	}
    else
    {
		printf("\nZEND parsing failed\n");
		RETURN_FALSE;
	}
}




PHP_FUNCTION(conf_set)
{
	dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
	int fd = -1, setbuf_len = 0,key_len,val_len;

	char *setbuf = NULL,*ret=NULL, *err=NULL,*val = "Set Success\n",*set_key,*set_value;


	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &set_key,&key_len,&set_value,&val_len) == SUCCESS)
	{
		if (dbcom_set_one(state, set_key, set_value, &ret,&err))
			if (err)
				val = "Set Failed\n";

		RETURN_STRING(val,1);
   }
   else
   {
   		printf("\nZEND parsing failed\n");
   		RETURN_FALSE;
	}

}


PHP_FUNCTION(conf_set_buffer)
{
	dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
	int fd = -1, setbuf_len = 0;
	char *setbuf = NULL,*ret=NULL, *err=NULL,*val = "Set Success\n";

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &setbuf,&setbuf_len) == SUCCESS)
	{
		if (dbcom_set_buffer(state, setbuf,  &ret,&err))
			if (err)
				val = "Set Failed\n";

		RETURN_STRING(val,1);
   }
   else
   {
   		printf("\nZEND parsing failed\n");
   		RETURN_FALSE;
	}

}


PHP_FUNCTION(conf_save)
{
	dbcom_handle state = dbcom_init(GETSOCK,SETSOCK, TRANS_LOCAL_TCP);
	char *ret = NULL, *err = NULL,*val="Vales saved\n";
	int ret_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &ret,&ret_len) == SUCCESS)
	{
		if (dbcom_save(state, &ret,&err))
			if (err)
				val = "Error in saving values\n";
		RETURN_STRING(val,1);
	}
	else
	{
		printf("\nZEND parsing failed\n");
		RETURN_FALSE;
	}

}



