/*
 * br_hook.h
 *
 * Copyright (c) Arada Systems, 1996-2007
 *
 */


/*************** Constants ***************/
#define MAX_RULES	8

/* Verdicts */
#define BR_CONTINUE	0
#define BR_ACCEPT	1
#define BR_DROP		2


/*************** Data types ***************/

struct bridge_rule {
	struct net_bridge	*br;
	struct net_device 	*indev;
	struct net_device 	*outdev;
	unsigned int		pkt_type;
	int			target;
	int			arg;
};

struct bridge_hook {
	char name[32];
	spinlock_t lock;
	int num_rules;
	struct bridge_rule br_rules[MAX_RULES];
};


/*************** Macros ***************/
#define PKT_TYPE_MATCH(match, pkt_type) \
({ \
	int ret; \
	if (match & 0xFFFF0000) { \
		if ((match & 0xFFFF) == pkt_type) \
 			ret = 0; \
 		else \
 			ret = 1; \
	} else { \
 		if ((match & 0xFFFF) == pkt_type) \
 			ret = 1; \
 		else \
 			ret = 0; \
 	} \
 	ret; \
})


/*************** Forward Definitions ***************/
extern int (*br_targets[])(struct sk_buff **pskb, int hook, int arg);
extern struct bridge_hook br_hooks[];

extern int add_br_hook_rule(int hook, int position, char *bridge, char *indev, char *outdev, 
				unsigned int pkt_type, int target, int arg);
extern int del_br_hook_rule(int hook, char *bridge, char *indev, char *outdev,
				unsigned int pkt_type, int target, int arg);
extern int get_next_br_hook_rule(int hook, int *cookie, char *bridge, char *indev, char *outdev,
				unsigned int *pkt_type, int *target, int *arg);

/*************** Functions ***************/
static inline int BR_HOOK(int hook, struct sk_buff **pskb, 
				struct net_device *indev, 
				struct net_device *outdev)
{
 	struct net_bridge_port *br_port = rcu_dereference((*pskb)->dev->br_port);
	struct net_bridge *br;
	struct bridge_hook *br_hook;
	struct bridge_rule *br_rule;
	int i, verdict = BR_CONTINUE;

	if (!br_port)
		return -1;
	br = br_port->br;

	br_hook = &br_hooks[hook];

	spin_lock(&br_hook->lock);

	for (i = 0; i < br_hook->num_rules; i++) {
		br_rule = &br_hook->br_rules[i];
		if ((!br_rule->br || br_rule->br == br) && 
			(!br_rule->indev || br_rule->indev == indev) &&
			(!br_rule->outdev || br_rule->outdev == outdev) &&
			(!br_rule->pkt_type || PKT_TYPE_MATCH((br_rule->pkt_type), (ntohs(eth_hdr(*pskb)->h_proto))))) {
			verdict = (*br_targets[br_rule->target])(pskb, hook, br_rule->arg);
		}
		if (verdict != BR_CONTINUE)
			break;
	}

	spin_unlock(&br_hook->lock);

	return verdict;
}

