/*
 *  ebt_rem1q
 *
 *  Copyright (c) Arada Systems, 1996-2007
 *
 */

#include <linux/module.h>
#include <linux/if_ether.h>
#include <linux/if_vlan.h>
#include <net/sock.h>
#include <linux/netfilter_bridge/ebtables.h>
#include "ebt_8021q.h"

static int ebt_target_rem1q(struct sk_buff **pskb, unsigned int hooknr,
   const struct net_device *in, const struct net_device *out,
   const void *data, unsigned int datalen)
{
	struct ebt_8021q_info *info = (struct ebt_8021q_info *)data;

	if (skb_shared(*pskb) || skb_cloned(*pskb)) {
		struct sk_buff *nskb;

		nskb = skb_copy(*pskb, GFP_ATOMIC);
		if (!nskb)
			return NF_DROP;
		if ((*pskb)->sk)
			skb_set_owner_w(nskb, (*pskb)->sk);
		kfree_skb(*pskb);
		*pskb = nskb;
	}

	/* Gobble up the VLAN header by moving the ethernet src & dst addresses
	 * to right by 4 bytes 
	 */
	memmove(eth_hdr(*pskb)->h_dest + VLAN_HLEN, eth_hdr(*pskb)->h_dest, (2 * ETH_ALEN));
	/* Eat 4 bytes at the head */
	skb_pull(*pskb, VLAN_HLEN);
	(*pskb)->mac.raw = (*pskb)->data;
	(*pskb)->protocol = eth_hdr(*pskb)->h_proto;
	
	return info->target;
}

static int ebt_target_rem1q_check(const char *tablename, unsigned int hookmask,
   const struct ebt_entry *e, void *data, unsigned int datalen)
{
	struct ebt_8021q_info *info = (struct ebt_8021q_info *)data;

	if (datalen != EBT_ALIGN(sizeof(struct ebt_8021q_info)))
		return -EINVAL;
	if (BASE_CHAIN && info->target == EBT_RETURN)
		return -EINVAL;
	CLEAR_BASE_CHAIN_BIT;
	if (strcmp(tablename, "nat"))
		return -EINVAL;
	if (hookmask & ~(1 << NF_BR_POST_ROUTING))
		return -EINVAL;
	if (INVALID_TARGET)
		return -EINVAL;
	return 0;
}

static struct ebt_target rem1q =
{
	.name		= EBT_REM1Q_TARGET,
	.target		= ebt_target_rem1q,
	.check		= ebt_target_rem1q_check,
	.me		= THIS_MODULE,
};

static int __init init(void)
{
	return ebt_register_target(&rem1q);
}

static void __exit fini(void)
{
	ebt_unregister_target(&rem1q);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
